import opennlp.tools.postag.POSModel;
import opennlp.tools.postag.POSTaggerME;
import opennlp.tools.sentdetect.SentenceModel;


public class POSTagger {

	private String caminhoArquivo;
	
	/**
	 * Construtor da classe.
	 * @param caminhoArquivo Caminho do arquivo modelo.
	 */
	public POSTagger(String caminhoArquivo){
		this.caminhoArquivo=caminhoArquivo;
	}
	
	/**
	 * Retorna os rotulos gramaticais da respectiva sentenca
	 * @param modelo Modelo (arquivo .bin) a ser utilizado
	 * @param sentenca Sentenca a ser rotulada.
	 * @return Lista de rotulos.
	 */
	public String[] getTags(String[] sentenca){
		POSTaggerME tagger = new POSTaggerME(this.getModelo());
		String[] posTags = tagger.tag(sentenca);
		return(posTags);
	}
	
	/**
	 * Retorna as probabilidades dos rotulos gramaticais da respectiva sentenca
	 * @param modelo Modelo (arquivo .bin) a ser utilizado
	 * @param sentenca Sentenca a ser rotulada.
	 * @return Lista de rotulos.
	 */
	public double[] getTagsProb(String[] sentenca){
		POSTaggerME tagger = new POSTaggerME(this.getModelo());
		String[] posTags = tagger.tag(sentenca);
		return(tagger.probs());
	}
		
	
	/**
	 * Retorna o modelo de POS.
	 * @param caminhoArquivoBIN Caminho ate o arquivo .bin (pt-sent.bin, por exemplo)
	 * @return Modelo de Sentença.
	 */
	public POSModel getModelo(){
		return (Util.getModeloPOS(caminhoArquivo));
	}
	
	
}
