
import java.lang.Iterable;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import opennlp.tools.postag.POSTaggerME;
import opennlp.tools.sentdetect.SentenceDetectorME;
import opennlp.tools.tokenize.TokenizerME;

/**
 * Classe que realiza o POS tagging.
 *
 */
public class Tagger {

    public String caminhoSentBIN;
    public String caminhoTokBIN;
    public String caminhoPOSBIN;

    /**
     * Construtor da classe.
     * @param caminhoSentBIN Caminho do arquivo .bin referente ao sentence detector.
     * @param caminhoTokBIN Caminho do arquivo .bin referente ao tokenizer.
     * @param caminhoPOSBIN Caminho do arquivo .bin referente ao POS Tagger.
     */
    public Tagger(String caminhoSentBIN, String caminhoTokBIN, String caminhoPOSBIN) {
        this.caminhoSentBIN = caminhoSentBIN;
        this.caminhoTokBIN = caminhoTokBIN;
        this.caminhoPOSBIN = caminhoPOSBIN;
    }

    public String getCaminhoSentBIN() {
        return caminhoSentBIN;
    }

    public String getCaminhoTokBIN() {
        return caminhoTokBIN;
    }

    public String getCaminhoPOSBIN() {
        return caminhoPOSBIN;
    }

    public void setCaminhoSentBIN(String caminhoSentBIN) {
        this.caminhoSentBIN = caminhoSentBIN;
    }

    public void setCaminhoTokBIN(String caminhoTokBIN) {
        this.caminhoTokBIN = caminhoTokBIN;
    }

    public void setCaminhoPOSBIN(String caminhoPOSBIN) {
        this.caminhoPOSBIN = caminhoPOSBIN;
    }

    /**
     * Rotula um texto retornando um conjunto de sentencas.
     * @param texto Texto a ser rotulado.
     * @return Conjunto de sentencas componentes do texto contendo conjuntos de tokens rotulados de acordo com seus respectivos POS tags.
     * @see Sentenca
     */

    public Sentenca[] tag(String texto, ArrayList<EntidadeMencionada> EMs) {


        Sentenca[] sentencas = null;

        String sent = this.getCaminhoSentBIN();//caminhos dos modelos treinados
        String tok = this.getCaminhoTokBIN();
        String pos = this.getCaminhoPOSBIN();



        SentenceDetectorME sentDet = new SentenceDetectorME(Util.getModeloSent(sent));// instancia a classe SentenceDetectorME que detecta as sentenças
        TokenizerME toke = new TokenizerME(Util.getModeloTok(tok));
        POSTaggerME post = new POSTaggerME(Util.getModeloPOS(pos));

        Pattern p1 = Pattern.compile("CATEG=\"(.*?)\"", Pattern.UNICODE_CASE);// cria essa expressao e as outras abaixo. Esta é para categoria.
        Pattern p2 = Pattern.compile("TIPOREL=\"(.*?)\"", Pattern.UNICODE_CASE);
        Pattern p3 = Pattern.compile("COREL=\"(.*?)\"", Pattern.UNICODE_CASE);
        Pattern p4 = Pattern.compile("TIPO=\"(.*?)\"", Pattern.UNICODE_CASE);
        Pattern p5 = Pattern.compile("ID=\"(.*?)\"", Pattern.UNICODE_CASE);


        //limpando onde não é EM do texto
        Pattern p = Pattern.compile("<(?!EM|/EM).*?>", Pattern.UNICODE_CASE);//cria expressao para encontrar tags que não sejam começo e fim de EN. Tags é tudo que está entre os sinais de < >
        Matcher m = p.matcher(texto);
        StringBuffer sb = new StringBuffer();// instanciando a StringBuffer
        boolean result = m.find();
        while (result) { //
            m.appendReplacement(sb, "");
            result = m.find();
        }
        m.appendTail(sb);
        texto = sb.toString();//insere na string texto o texo sem as marcações de entidades.
        
        ArrayList<Integer> posicoesEnts = new ArrayList<Integer>();// lista que contem o inicio de todas as entidades do texto. Inicio de entidade é: quantas palavras foram contadas ate chegar no inicio da EN.
        ArrayList<Integer> posicoesFimEnts = new ArrayList<Integer>();// .......... Fim da EN significa quantas palavras foram contadas até chegar no FIM de cada EN.
		// as linhas acima foram criadaS porque como precisou-se remover as indicações de EN, depois precisou-se saber onde se encontravam essas EN no texto.

//modificado 5 Agosto
        texto = texto.replace(">","%$%");
	texto = texto.replace("<","$%$");
//fim modificado 5 Agosto




        p = Pattern.compile("<.*?>", Pattern.UNICODE_CASE);//procura onde tem essas EN no texto.
        m = p.matcher(texto);
        sb = new StringBuffer();
        result = m.find();
        boolean comecoFim = true;
        while (result) { 
            
            m.appendReplacement(sb, " ");//apagou o sinal de < > que indicava a marcação de EN, para ter o texto limpo para poder encontrar o POS das palavras.

            String[] tokens = toke.tokenize(sb.toString().replace('\n', ' ')); //divide o texto em palavras para poder pegar o índice da palavra onde começa e termina a EN, para depois poder colocar a indicação da EN. A fim de não se executar o POS sobre os caracteres que indicam o início e o fim de uma EN (<EN> </EN>).
            
			if(comecoFim){
				posicoesEnts.add(tokens.length); //ADICIONA NAS LISTAS AS POSIÇÕES DE COMEÇO E FIM DAS EN.
			} else {
				posicoesFimEnts.add(tokens.length);
			}
            
        	comecoFim = !comecoFim;
            result = m.find();//encontra a próxima marcação de EN: (<EN> </EN>).
        }
        m.appendTail(sb);
        String textoSemEM = sb.toString();//texto sem a maracação da EN.
		
        p = Pattern.compile("<EM.*?>", Pattern.UNICODE_CASE); // encontrar informações como categoria, tipo de EN. p é a expressão regular.
        m = p.matcher(texto); //m é o objeto que utiliza a expressão regular.
        sb = new StringBuffer(); //sb é a string onde se coloca os resultados da pesquisa.
        result = m.find(); //encontra a próxima ocorrência da expressão regular.
        
		Iterator<Integer> it = posicoesEnts.iterator();//it é um objeto que percorre a lista posicoesEnts para colocá-las no vetor de EN chamado EMs.
		Iterator<Integer> it2 = posicoesFimEnts.iterator();
		
        while (result) {
        	int inicio = it.next();//pega as próximas posições de começo de EN e 
        	int fim = it2.next();//... e fim de EN.
        	
            
            String categ = "";
            String tiporel = "";
            String corel = "";
            String tipo = "";
            String id = "";
            
            Matcher mat;//inicializando um utilizador de expressão regular de nome Matcher.
            
            mat = p1.matcher(m.group());//executa cada uma das expressões regulares criadas nas linhas 79 à 83. Este laço é a expressão regular para categorias.
            mat.find();
            if (!mat.hitEnd()) {
                categ = mat.group();
            }
            mat = p2.matcher(m.group());// Este laço é a expressão regular para tipo relação.
            mat.find();
            if (!mat.hitEnd()) {
                tiporel = mat.group();
            }
            mat = p3.matcher(m.group());
            mat.find();
            if (!mat.hitEnd()) {
            	corel = mat.group();
            }                
            mat = p4.matcher(m.group());// Este laço é a expressão regular para tipo.
            mat.find();
            if (!mat.hitEnd()) {
            	tipo = mat.group();
            }
            mat = p5.matcher(m.group());//Este laço é a expressão regular para ID, pois cad EN tem uma identificação.
            mat.find();
            if (!mat.hitEnd()) {
            	id = mat.group();// mat.group é uma função que retorna o que foi encontrado pela expressão regular. Neste laço é o id.
            }

	        EMs.add(new EntidadeMencionada(inicio, fim-1, id, categ, tipo, tiporel, corel));//adiciona as informaçoes do parentese na EN e coloca num vetor de EN chamado EMs.
            
            result = m.find(); //encontra a próxima ER.
        }
        m.appendTail(sb); //para finalizar a string.
        texto = sb.toString();//adicionando a própria string nela mesma.


        String[] sentences = sentDet.sentDetect(textoSemEM.replace('\n', ' '));//replace é uma função que significa trocar algo. Agora se tem uma grande string como todo o texto do HAREM corrido. Foram retirada as novas linhas e com o detector de sentenças encontrou-se um vetor de sentenças, foram trocadas as novas linhas por espaços.
        			//sentences é um vetor de strings.
        sentencas = new Sentenca[sentences.length]; //sentencas é um vetor de objetos.
        
        String[] tokens; //vetor de nome tokens.
        	
        //laço que separa as sentenças em palavras.
        for (int cont = 0; cont < sentences.length; cont++) {
            Sentenca sentenca = new Sentenca(); //criou-se um objeto de nome sentença que por enquanto está vazio.
            tokens = toke.tokenize(sentences[cont]); //separa as sentenças em palavras. Ex. [.....'da','PUC','.',....]


            // das linhas 192 ate a 205 é excluido os tokens vazios do vetor.
            int aux = 0;
            for (int i = 0; i < tokens.length; i++) {
                if (tokens[i].equals("")) { // .equals funçao que compara dois objetos.
                    aux++;
                }
            }
            String[] tokensUteis = new String[tokens.length - aux]; //instaciou um outro vetor chamado tokensUteis que terá os token, ou seja, qualquer palavra e pontuações das sentenças sem os vazios.
            aux = 0;
            for (int i = 0; i < tokens.length; i++) {
                if (!tokens[i].equals("")) {
                    tokensUteis[aux] = tokens[i];
                    aux++; //adiciona sempre na próxima posição do vetor.
                }
            }

            String[] tags = post.tag(tokensUteis); //executa o POS tagging. Criou um vetor chamado tags.

            for (int contt = 0; contt < tokensUteis.length; contt++) {//tem-se dois vetores que aqui é feita a indicação de cada palavras com o seu POS correspondente.
                Token token = new Token();
                token.setSimbolo(tokensUteis[contt]); //simbolo pode ser a palavra ou a pontuação.
                token.setPOStag(tags[contt]);
                sentenca.addToken(token); //adiciona o objeto com o POS e a palavra.
            }
            sentencas[cont] = sentenca; //vetor de nome sentencas que adiciona a sentença atual para ele.
        }
        return (sentencas); //retorna o vetor e termina a função.
    }

    /**
     * Converte sentencas em Strings seguindo o formato "simbolo<POStag>" (ex.: carro<N>).
     * @param sentencas Conjunto de sentencas.
     * @return Conjunto de sentencas.
     */
    public String[] sentencesToString(Sentenca[] sentencas) {
        String[] sentences = new String[sentencas.length];

        for (int cont = 0; cont < sentencas.length; cont++) {
            String sent = "";
            Sentenca s = sentencas[cont];
            for (int cont2 = 0; cont2 < s.getTokens().size(); cont2++) {
                String token = s.getToken(cont2).getSimbolo();
                token += "<" + s.getToken(cont2).getPOStag() + ">, ";
                sent += token;
            }
            sentences[cont] = sent;
        }
        return (sentences);
    }
}
