import opennlp.tools.sentdetect.SentenceModel;
import opennlp.tools.tokenize.TokenizerME;
import opennlp.tools.tokenize.TokenizerModel;
import opennlp.tools.util.Span;

/**
 * Classe que implementa a quebra de sentencas em tokens (fichas) representadas pelos simbolos componentes da sentenca.
 * Utiliza-se da biblioteca OpenNLP (http://incubator.apache.org/opennlp/) que por sua vez
 * utiliza-se de processamento estatistico (Maxima Entropia).
 */
public class Tokenizer {

	private String caminhoArquivoBIN;
	
	/**
	 * Construtor da classe.
	 * @param caminhoArquivoBIN Caminho do arquivo .bin (en-token.bin, por exemplo).
	 */
	public Tokenizer(String caminhoArquivoBIN){
		this.caminhoArquivoBIN = caminhoArquivoBIN;
		System.out.println("Construindo Tokenizer...");
	}
	
	
	public String getCaminhoArquivoBIN() {
		return caminhoArquivoBIN;
	}
	public void setCaminhoArquivoBIN(String caminhoArquivoBIN) {
		this.caminhoArquivoBIN = caminhoArquivoBIN;
	}
	
	
	/**
	 * Divide o texto passado como parâmetro em tokens.
	 * @param modelo Modelo a ser utilizado (pt-tokenizer.bin, por exemplo)
	 * @param texto Texto a ser quebrado. 
	 * @return Lista de palavras.
	 */
	public String[] getTokens(String texto){
		TokenizerModel modelo = this.getModelo(this.caminhoArquivoBIN);
		TokenizerME tk = new TokenizerME(modelo);
		return(tk.tokenize(texto));
	}
	
	/**
	 * Divide o texto passado como parâmetro em tokens.
	 * @param modelo Modelo a ser utilizado (pt-tokenizer.bin, por exemplo)
	 * @param texto Texto a ser quebrado. 
	 * @return Lista contendo indicadores de inicio e fim das fichas.
	 */
	public Span[] getTokensSpan(String texto){
		TokenizerModel modelo = this.getModelo(this.caminhoArquivoBIN);
		TokenizerME tk = new TokenizerME(modelo);
		return(tk.tokenizePos(texto));
	}
	
	/**
	 * Retorna a probabilidade de cada um dos tokens.
	 * @param modelo Modelo a ser utilizado (pt-tokenizer.bin, por exemplo)
	 * @param texto Texto a ser quebrado. 
	 * @return Lista contendo as probabilidades das fichas.
	 */
	public double[] getTokensProb(String texto){
		TokenizerModel modelo = this.getModelo(this.caminhoArquivoBIN);
		TokenizerME tk = new TokenizerME(modelo);
		tk.tokenize(texto);
		return(tk.getTokenProbabilities());
	}
	
	/**
	 * Retorna o modelo de sentença.
	 * @param caminhoArquivoBIN Caminho ate o arquivo .bin (pt-sent.bin, por exemplo)
	 * @return Modelo de Sentença.
	 */
	public TokenizerModel getModelo(String caminhoArquivoBIN){
		return (Util.getModeloTok(caminhoArquivoBIN));
	}
}
