import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import opennlp.tools.namefind.TokenNameFinderModel;
import opennlp.tools.postag.POSModel;
import opennlp.tools.sentdetect.SentenceModel;
import opennlp.tools.tokenize.TokenizerModel;

/**
 * Esta classe agrega as funcionlidades de criacao dos modelos utilizadas pelas demais classes do pacote em questao.
 *
 */

public class Util {
	
	/**
	 * Retorna o modelo de sentença.
	 * @param caminhoArquivoBIN Caminho ate o arquivo .bin (pt-sent.bin, por exemplo)
	 * @return Modelo de Sentença.
	 */
	public static SentenceModel getModeloSent(String caminhoArquivoBIN){
		try{
			InputStream modeloEntrada = new FileInputStream(caminhoArquivoBIN);
			SentenceModel modelo = new SentenceModel(modeloEntrada);
			modeloEntrada.close();
			return(modelo);
		}
		catch (IOException e){
			e.printStackTrace();
			return(null);
		}
	}
	
	/**
	 * Retorna o modelo de tokens.
	 * @param caminhoArquivoBIN Caminho ate o arquivo .bin (pt-sent.bin, por exemplo)
	 * @return Modelo de Sentença.
	 */
	public static TokenizerModel getModeloTok(String caminhoArquivoBIN){
		try{
			InputStream modeloEntrada = new FileInputStream(caminhoArquivoBIN);
			TokenizerModel modelo = new TokenizerModel(modeloEntrada);
			modeloEntrada.close();
			return(modelo);
		}
		catch (IOException e){
			e.printStackTrace();
			return(null);
		}
	}
	
	/**
	 * Retorna o modelo de tokens.
	 * @param caminhoArquivoBIN Caminho ate o arquivo .bin (pt-sent.bin, por exemplo)
	 * @return Modelo de Sentença.
	 */
	public static POSModel getModeloPOS(String caminhoArquivoBIN){
		try{
			InputStream modeloEntrada = new FileInputStream(caminhoArquivoBIN);
			POSModel modelo = new POSModel(modeloEntrada);
			modeloEntrada.close();
			return(modelo);
		}
		catch (IOException e){
			e.printStackTrace();
			return(null);
		}
	}
}
